#include "opencv2/highgui/highgui.hpp"
#include "opencv2/imgproc/imgproc.hpp"
#include <iostream>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <sstream>

#define PI 3.141592653589793

using namespace std;
using namespace cv;


/*
	Prevedie HSV(0..359,0..100,0..100) do HSV(0..179,0..255,0..255), ktory sa vyuziva v openCV
*/
Scalar getHSV(int hue_0_360, int sat_0_100, int val_0_100){
	return Scalar(hue_0_360/2, 255*sat_0_100/100, 255*val_0_100/100);
}

/*String intToStr(int i){
	char buffer [50];
  
	itoa(i,buffer,10);
	return buffer;
}*/

string intToStr(int i){
	stringstream a;
	a<< i;
	string si;
	a >> si;
	return si;
}
/*
	Nakresli do obrazku polygon a jeho nazov
*/
void drawShape(vector<Point> vector, Mat& img, String label){
	for (int i = 0; i < vector.size() -1; i++){
		line(img, vector.at(i), vector.at(i+1), CV_RGB(255,0,100), 1, 8, 0);
		putText(img, intToStr(i+1), vector.at(i), FONT_HERSHEY_PLAIN, 0.6, CV_RGB(255,0,100));
	}
	line(img, vector.at(0), vector.at(vector.size()-1), CV_RGB(255,0,100), 1, 8, 0);
	putText(img, intToStr(vector.size()-1+1), vector.at(vector.size()-1), FONT_HERSHEY_PLAIN, 0.6, CV_RGB(255,0,100));
}
/*
	Vypocita double vzdialenost dvoch bodov v 2D
*/
double dist(double x1, double y1, double x2, double y2){
	return sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
}

double getCenter(double p1, double p2){
	return (p1+p2)/2;
}

Point getCenterPoint(Point p1, Point p2){
  Point p;
  p.x = getCenter(p1.x,p2.x);
  p.y = getCenter(p1.y,p2.y);
  return p;
}

Point getCenterPoint(vector<Point> v){
  Point p;
  double x=0;
  double y=0;
  for(int i=0; i<v.size(); i++){
	x = x+v.at(i).x;
	y = y+v.at(i).y;
  }
  p.x = x/v.size();
  p.y = y/v.size();
  return p;
}

double getLongestTriangleEdgeLength(vector<Point>& triangle){
	double p1x = triangle.at(0).x;
	double p1y = triangle.at(0).y;

	double p2x = triangle.at(1).x;
	double p2y = triangle.at(1).y;

	double p3x = triangle.at(2).x;
	double p3y = triangle.at(2).y;

	double dst1_2 = dist(p1x,p1y,p2x,p2y);
	double dst1_3 = dist(p1x,p1y,p3x,p3y);
	double dst2_3 = dist(p2x,p2y,p3x,p3y);

	// najdeme najdlhsiu hranu trouholnika
	double* longest = &dst1_2;
	if (dst1_3 > *longest) longest = &dst1_3;
	if (dst2_3 > *longest) longest = &dst2_3;

	return *longest;
}

vector<Point> getLongestTriangleEdge(vector<Point>& triangle){
	double p1x = triangle.at(0).x;
	double p1y = triangle.at(0).y;

	double p2x = triangle.at(1).x;
	double p2y = triangle.at(1).y;

	double p3x = triangle.at(2).x;
	double p3y = triangle.at(2).y;

	double dst1_2 = dist(p1x,p1y,p2x,p2y);
	double dst1_3 = dist(p1x,p1y,p3x,p3y);
	double dst2_3 = dist(p2x,p2y,p3x,p3y);
  
  vector<Point> result;
  double longest =  getLongestTriangleEdgeLength(triangle);
  if (longest==dst1_2){
	result.push_back(Point(p1x,p1y));
	result.push_back(Point(p2x,p2y));
  }else if (longest==dst1_3){
	result.push_back(Point(p1x,p1y));
	result.push_back(Point(p3x,p3y));
  }else if (longest==dst2_3){
	result.push_back(Point(p2x,p2y));
	result.push_back(Point(p3x,p3y));
  }
  return result;
}

int getMutualYposition(double pointX, double pointY, double p1x, double p1y, double p2x, double p2y){
	double k = (p2y-p1y)/(p2x-p1x);
	double q = p1y-k*p1x;
	double result = (k*pointX+q);
	if (result>pointY) return +1;
	if (result<pointY) return -1;
	return 0;
}

int getMutualYposition(Point2d point, Point2d p1, Point2d p2){
	return getMutualYposition(point.x, point.y, p1.x, p1.y, p2.x, p2.y);
}

/*
	Vrati uhol robota podla zadanych bodov trojuholnika. hore +0, vpravo +90, dole +180, vlavo -90
*/
double getRobotAngle(vector<Point>& triangle){

	double p1x = triangle.at(0).x;
	double p1y = triangle.at(0).y;

	double p2x = triangle.at(1).x;
	double p2y = triangle.at(1).y;

	double p3x = triangle.at(2).x;
	double p3y = triangle.at(2).y;

	double dst1_2 = dist(p1x,p1y,p2x,p2y);
	double dst1_3 = dist(p1x,p1y,p3x,p3y);
	double dst2_3 = dist(p2x,p2y,p3x,p3y);

	double longest = getLongestTriangleEdgeLength(triangle);

	double angle=0;
	double leftX,rightX,leftY,rightY;

	// defaultne predpokladame vrchol oproti najdlhsej strane nad najdlhsou hranou
	bool Yswitch=false;

	if (longest==dst1_2){
		if (p1x<p2x){
			leftX = p1x; rightX = p2x;
			leftY = p1y; rightY = p2y;
		}else{
			leftX = p2x; rightX = p1x;
			leftY = p2y; rightY = p1y;
		}
		
		if (getMutualYposition(p3x,p3y,leftX,leftY,rightX,rightY)==-1) Yswitch=true;
		//if (getCenter(leftY,rightY)<p3y) Yswitch=true; // stred najdlhsej hrany VS treti bod

		cout << "longest edge: 1-2\n";
	}else if (longest==dst1_3){
		if (p1x<p3x){
			leftX = p1x; rightX = p3x;
			leftY = p1y; rightY = p3y;
		}else{
			leftX = p3x; rightX = p1x;
			leftY = p3y; rightY = p1y;
		}

		if (getMutualYposition(p2x,p2y,leftX,leftY,rightX,rightY)==-1) Yswitch=true;
		//if (getCenter(leftY,rightY)<p2y) Yswitch=true; // stred najdlhsej hrany VS treti bod

		cout << "longest edge: 1-3\n";
	}else if (longest==dst2_3){
		if (p2x<p3x){
			leftX = p2x; rightX = p3x;
			leftY = p2y; rightY = p3y;
		}else{
			leftX = p3x; rightX = p2x;
			leftY = p3y; rightY = p2y;
		}

		if (getMutualYposition(p1x,p1y,leftX,leftY,rightX,rightY)==-1) Yswitch=true;
		//if (getCenter(leftY,rightY)<p1y) Yswitch=true; // stred najdlhsej hrany VS treti bod

		cout << "longest edge: 2-3\n";
	}

  if (rightX-leftX==0) angle=1.570796327;
  else angle=atan((rightY-leftY)/(rightX-leftX));

	if (Yswitch){
		// preklopenie o PI/2
		if (angle<0) angle = PI+angle;
		else angle = -PI+angle;
		cout << "Yswitch \n";
	}

	return angle;
}

double radToDeg(double rad){
	return rad * (180/PI);
}

/*
	Modifikovana funkcia z internetu
*/
vector<Point> getTriangleVertices(const vector< vector<Point> >& contours, int minTriangleArea=0)
{
	vector<Point> approxTriangle;
	vector<Point> allTriangleVertices;
	// obsah sipky je 450-480
	for(size_t i = 0; i < contours.size(); i++)
	{
		if (fabs(contourArea(contours[i])) < minTriangleArea) continue;
		approxPolyDP(contours[i], approxTriangle, arcLength(Mat(contours[i]), true)*0.05, true);
		if(approxTriangle.size() == 3)
		{
			copy(approxTriangle.begin(), approxTriangle.end(), back_inserter(allTriangleVertices));
		}
	}
	return allTriangleVertices;
}

vector<Point> findArrow(Mat imageRGB, Scalar minHSV, Scalar maxHSV, int minTriangleArea, string label=""){
	Mat imageHSV;
	Mat filtered;
	cvtColor(imageRGB, imageHSV, CV_BGR2HSV);
	inRange(imageHSV, minHSV, maxHSV, filtered);

	if (label!=""){
		namedWindow(label, WINDOW_AUTOSIZE );
		imshow(label, filtered);
	}

	vector<vector<Point> > contours;
	vector<Vec4i> hierarchy;
	findContours(filtered.clone(), contours, hierarchy, CV_RETR_TREE, CV_CHAIN_APPROX_SIMPLE, Point(0, 0));

	vector<Point> arrow = getTriangleVertices(contours, minTriangleArea);

	return arrow;
}